#include <stdio.h>
#include "global.h"

#define INCR 20

report_range()
{
  struct FPRange drange;
  struct Range range;
  char buff[1024], buff2[300];

  if(G_read_fp_range(name, G_mapset(), &drange)<=0)
  {
     sprintf(buff, "Can't read f_range for map %s", name);
     G_fatal_error(buff);
  }
  G_get_fp_range_min_max(&drange, &old_dmin, &old_dmax);
  if(G_is_d_null_value(&old_dmin) || G_is_d_null_value(&old_dmax))
      printf ("Data range is empty\n");
  else
  {
      sprintf(buff, "%.10f",old_dmin);
      sprintf(buff2, "%.10f",old_dmax);
      G_trim_decimal(buff);
      G_trim_decimal(buff2);
      printf ("Data range of %s is %s to %s\n", name, buff, buff2);
  }

  if(G_read_range(name, G_mapset(), &range)<=0)
  {
     sprintf(buff, "Can't read range for map %s", name);
     G_fatal_error(buff);
  }
  G_get_range_min_max(&range, &old_min, &old_max);
  if(G_is_c_null_value(&old_min) || G_is_c_null_value(&old_max))
      printf ("Integer data range of %s is empty\n", name);
  else
      printf ("Integer data range of %s is %d to %d\n", 
		   name, (int)old_min, (int)old_max);
}

read_rules ()
{
   char buf[1024];
   DCELL dLow, dHigh;
   CELL iLow, iHigh;
   int line, n, nrules=0;

   report_range();
   printf("\nEnter the rule or 'help' for the format description:\n");
   G_quant_init(&quant_struct);
   for (line=1;;line++)
   {
      if (isatty(0)) printf ("> ");
      if (!gets(buf)) return 0;
      for (n=0;buf[n];n++)
	   if(buf[n] == ',')
	      buf[n] = ' ';
      G_strip(buf);
      if (*buf == 0) continue;
      if (*buf == '#') continue;
      if (strcmp(buf, "end") == 0) break;

      if (strcmp(buf, "help") == 0)
      {
	  printf ("Enter a rule in one of these formats:\n");
	  printf("float_low:float_high:int_low:int_high\n");
	  printf("float_low:float_high:int_val  (i.e. int_high == int_low)\n");
	  printf("*:float_val:int_val           (interval [inf, float_val])\n");
	  printf("float_val:*:int_val           (interval [float_val, inf])\n");
      }	       

      /* we read and record into quant table all values, even int as doubles
	 we convert the range and domain values to the right format when we 
	 lookup the values in the quant table */
      switch (sscanf (buf, "%lf:%lf:%d:%d", &dLow, &dHigh, &iLow, &iHigh))
      {
        case 3:  G_quant_add_rule(&quant_struct, dLow, dHigh, iLow, iLow);
		 nrules++;
		 break;

        case 4:  G_quant_add_rule(&quant_struct, dLow, dHigh, iLow, iHigh);
		 nrules++;
		 break;

        default:
	    if(sscanf (buf, "%lf:*:%d", &dLow, &iLow)==2)
	    {
	       G_quant_set_pos_infinite_rule(&quant_struct, dLow, iLow);
	       nrules++;
            }	

	    else if(sscanf (buf, "*:%lf:%d", &dHigh, &iLow)==2) 
	    {
	       G_quant_set_neg_infinite_rule(&quant_struct, dHigh, iLow);
	       nrules++;
            }

	    else
	       printf("%s is not a valid rule\n", buf);
	    break;
      } /* switch */
    } /* loop */

  return nrules;
}

